/* $Id: mga_common.h,v 1.5 1998/11/02 04:39:04 brian Exp $
******************************************************************************

   Matrox -- common driver structures and definitions

   Copyright (C) 1997		Jonas Borgstrm
   Copyright (C) 1997		Hartmut Niemann
   Copyright (C) 1997           Matthias Grimrath

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/


#ifndef _MGA_COMMON_H_
#define _MGA_COMMON_H_

#include <Matrox/mga_common_lib.h>

#ifndef PCI_DEVICE_ID_MATROX_MIL_2
#  define PCI_DEVICE_ID_MATROX_MIL_2 0x051B
#endif

#ifndef PCI_DEVICE_ID_MATROX_MIL_2_AGP
#  define PCI_DEVICE_ID_MATROX_MIL_2_AGP 0x051F
#endif

/*
** Structs
*/
struct common_private { /* Vars common to both Millenium and Mystique */
	struct kgi_chipset chipinfo;
	
	struct mem_region framebuffer;
	struct mem_region aperture;
	struct kgi_mmio_region graphic_mmio_linear_fb;

	struct wait_queue *vsync_wait;
	
	kgiu8   irq;
	kgiu8   rev;
	kgiu8   bus;
	kgiu8   dev_fn;
	kgiu8   coldinit;
	kgiu32  chip_id;
	
	kgi_uint origin, org_xdot, org_ydot, org_offs;
        kgi_uint frame;
	kgi_uint split;
	kgi_uint fb_size; /* # of pixels NOT bytes */
	kgi_uint bpp;

        kgiu32  ptr_old, cur_old;
        kgiu16  ptr_and, cur_and;
        kgiu16  ptr_xor, cur_xor;
};

/* 
** Convenience (?) macros 
*/
#define MGA_APERTURE    (private->cp.aperture)
#define MGA_FRAMEBUFFER (private->cp.framebuffer)
#define MGA_BASE1       (MGA_APERTURE.baseptr)     /* control aperture */
#define MGA_BASE2       (MGA_FRAMEBUFFER.baseptr)  /* frame buffer     */
#define MGA_IRQ         (private->cp.irq)
#define MGA_PCI_BUS     (private->cp.bus)
#define MGA_PCI_DEV_FN  (private->cp.dev_fn)
#define MGA_CHIP_ID     (private->cp.chip_id)
#define MGA_COLDINIT    (private->cp.coldinit)
#define MGA_CP          (private->cp)

/*
** Imported symbols. These are supposed to be defined by the *_main files.
*/

extern const struct kgi_chipset chipinfo;

extern const struct mem_region mga_aperture;
extern const struct mem_region mga_framebuffer;
extern const struct kgi_mmio_region graphic_mmio_linear_fb;

extern const char chipset_name_init[]; /* Print on klog on driver init */

void adjust_timing(struct kgi_mode *mode);
int adjust_virtual(struct kgi_mode *mode);
void mga_interrupt_handler(int irq, void *d, struct pt_regs *regs);

/*
** Prototypes
*/
int mga_chipset_init(struct kgi_card *card, struct kgi_display *dpy,
		     int privsize, int productID);
int mga_cold_init(struct kgi_display *dpy);
void mga_cold_shutdown(struct kgi_display *dpy);
/* Please have a look at the comment in mga_common.c */
void mga_chipset_done(struct kgi_display *dpy);

/* Card specific includes */
#if defined(DRIVER_mga1064)
#include "Matrox/mga1064_main.h"
#elif defined(DRIVER_mga2064) || defined(DRIVER_mga2164)
#include "Matrox/mga2x64_main.h"
#else
#warning include the headerfile for the new Matrox card here
#endif

#define GET_PRIVATE \
    struct chipset_private *private = KGIM_PRIVATE(dpy, chipset)

#endif /* _MGA_COMMON_H_ */
